﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Diagnostics;

namespace howto_compare_sorts
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // If the Sorted box is checked, enable the % Unsorted text box.
        private void chkSorted_CheckedChanged(object sender, EventArgs e)
        {
            txtNumUnsorted.Enabled = (chkSorted.Checked);
            lblPercentUnsorted.Enabled = txtNumUnsorted.Enabled;
        }

        // A random number generators used by various algorithms.
        private Random Rand = new Random();

        // The data to sort.
        private int[] Values = null, SortedValues = null;
        private int NumItems, MaxValue;

        // Prepare the data to sort.
        private void PrepareData()
        {
            // Allocate space.
            NumItems = int.Parse(txtNumItems.Text);
            Values = new int[NumItems];
            SortedValues = new int[NumItems];

            // Make random values.
            MaxValue = int.Parse(txtMaxValue.Text);
            for (int i = 0; i < NumItems; i++)
            {
                Values[i] = Rand.Next(0, MaxValue + 1);
            }

            // Sort the values if necessary.
            if (chkSorted.Checked)
            {
                Array.Sort(Values);

                // Unsort some of the values.
                int num_unsorted = int.Parse(txtNumUnsorted.Text);
                for (int i = 0; i < num_unsorted; i++)
                {
                    // Pick two items to swap.
                    int index1 = Rand.Next(0, NumItems);
                    int index2 = Rand.Next(0, NumItems);
                    int temp = Values[index1];
                    Values[index1] = Values[index2];
                    Values[index2] = temp;
                }
            }
        }

        // Sort with Bubblesort.
        private void btnBubblesort_Click(object sender, EventArgs e)
        {
            RunAlgorithm(txtTimeBubblesort, Bubblesort);
        }

        // Sort with Array.Sort.
        private void btnArraySort_Click(object sender, EventArgs e)
        {
            RunAlgorithm(txtTimeArraySort, ArraySort);
        }

        // Sort with Selectionsort.
        private void btnSelectionsort_Click(object sender, EventArgs e)
        {
            RunAlgorithm(txtTimeSelectionsort, Selectionsort);
        }

        // Sort with Quicksort.
        private void btnQuicksort_Click(object sender, EventArgs e)
        {
            RunAlgorithm(txtTimeQuicksort, Quicksort);
        }

        // Sort with Countingsort.
        private void btnCountingsort_Click(object sender, EventArgs e)
        {
            RunAlgorithm(txtTimeCountingsort, Countingsort);
        }

        // Run an algorithm.
        private delegate void Algorithm(int[] values);
        private void RunAlgorithm(TextBox time_textbox, Algorithm algorithm)
        {
            time_textbox.Clear();
            Cursor = Cursors.WaitCursor;
            Refresh();
            int num_trials = int.Parse(txtNumTrials.Text);

            // Prepare the data.
            PrepareData();

            Stopwatch watch = new Stopwatch();
            for (int trial = 0; trial < num_trials; trial++)
            {
                // Copy the unsorted data into the SortedValues array.
                Array.Copy(Values, SortedValues, NumItems);

                // Run the algorithm.
                watch.Start();
                algorithm(SortedValues);
                watch.Stop();
            }

            // Display the total time.
            time_textbox.Text = watch.Elapsed.TotalSeconds.ToString("0.00") + " sec";

            // Verify that the algorithm worked.
            Verify();

            Cursor = Cursors.Default;
        }

        // Verify that the values are sorted.
        private void Verify()
        {
            for (int i = 1; i < NumItems; i++)
            {
                if (SortedValues[i] < SortedValues[i - 1])
                {
                    MessageBox.Show("The values are not sorted.\n" +
                        "SortedValues[" + i + "] = " + SortedValues[i] +
                        " < " + "SortedValues[" + (i - 1) + "] = " + SortedValues[i - 1],
                        "Sorting Error", MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                    return;
                }
            }
        }

        #region Algorithms

        // Sort with Array.Sort.
        private void ArraySort(int[] values)
        {
            // Sort.
            Array.Sort(values);
        }

        // Bubblesort with:
        //   - Alternating upward and downward passes
        //   - Holding bubbled item in a temporary variable
        //   - Updating min and max to narrow the search range
        private void Bubblesort(int[] values)
        {
            int min = 0;
            int max = NumItems - 1;
            while (min < max)
            {
                // Bubble up.
                int last_swap = min - 1;
                // For i = min + 1 To max
                int i = min + 1;
                while (i <= max)
                {
                    // Find a bubble.
                    if (values[i - 1] > values[i])
                    {
                        // See where to drop the bubble.
                        int tmp = values[i - 1];
                        int j = i;
                        do
                        {
                            values[j - 1] = values[j];
                            j++;
                            if (j > max) break;
                        } while (values[j] < tmp);
                        values[j - 1] = tmp;
                        last_swap = j - 1;
                        i = j + 1;
                    }
                    else
                    {
                        i++;
                    }
                }
                // Update max.
                max = last_swap - 1;

                // Bubble down.
                last_swap = max + 1;
                // For i = max - 1 To min Step -1
                i = max - 1;
                while (i >= min)
                {
                    // Find a bubble.
                    if (values[i + 1] < values[i])
                    {
                        // See where to drop the bubble.
                        int tmp = values[i + 1];
                        int j = i;
                        do
                        {
                            values[j + 1] = values[j];
                            j--;
                            if (j < min) break;
                        } while (values[j] > tmp);
                        values[j + 1] = tmp;
                        last_swap = j + 1;
                        i = j - 1;
                    }
                    else
                    {
                        i--;
                    }
                }
                // Update min.
                min = last_swap + 1;
            }
        }

        // Selectionsort.
        private void Selectionsort(int[] values)
        {
            for (int i = 0; i < NumItems - 1; i++)
            {
                int best_value = values[i];
                int best_j = i;
                for (int j = i + 1; j < NumItems; j++)
                {
                    if (values[j] < best_value)
                    {
                        best_value = values[j];
                        best_j = j;
                    }
                }
                values[best_j] = values[i];
                values[i] = best_value;
            }
        }

        // Quicksort.
        private void Quicksort(int[] values)
        {
            Quicksort(values, 0, NumItems - 1);
        }

        // Use Quicksort to recursively sort
        // the items with indexes min <= i <= max.
        private void Quicksort(int[] values, int min, int max)
        {
            // If the list has no more than 1 element, it's sorted.
            if (min >= max) return;

            // Pick a dividing item.
            int i = Rand.Next(min, max + 1);
            int med_value = values[i];

            // Swap it to the front so we can find it easily.
            values[i] = values[min];

            // Move the items smaller than this into the left
            // half of the list. Move the others into the right.
            int lo = min;
            int hi = max;
            for (; ; )
            {
                // Look down from hi for a value < med_value.
                while (values[hi] >= med_value)
                {
                    hi--;
                    if (hi <= lo) break;
                }
                if (hi <= lo)
                {
                    values[lo] = med_value;
                    break;
                }

                // Swap the lo and hi values.
                values[lo] = values[hi];

                // Look up from lo for a value >= med_value.
                lo++;
                while (values[lo] < med_value)
                {
                    lo++;
                    if (lo >= hi) break;
                }
                if (lo >= hi)
                {
                    lo = hi;
                    values[hi] = med_value;
                    break;
                }

                // Swap the lo and hi values.
                values[hi] = values[lo];
            }

            // Sort the two sublists
            Quicksort(values, min, lo - 1);
            Quicksort(values, lo + 1, max);
        }

        // Countingsort.
        private void Countingsort(int[] values)
        {
            // Create the Counts array.
            int[] counts = new int[MaxValue + 1];

            // Count the items.
            for (int i = 0; i < NumItems; i++)
            {
                counts[values[i]]++;
            }

            // Place items in the sorted array.
            int index = 0;
            for (int i = 0; i <= MaxValue; i++)
            {
                for (int j = 0; j < counts[i]; j++)
                {
                    values[index++] = i;
                }
            }
        }

        #endregion Algorithms

    }
}
